/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLInputElement.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/GdomHTMLInputElement.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateGdomHTMLInputElement.h"
#include "gdom/PrivateIntGdomHTMLInputElement.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLInputElement::GDOMHTMLInputElement(PassRefPtr<WebCore::HTMLInputElement> impl)
    : GDOMHTMLElement(impl)
{
}

void
gdom_html_input_element_select (GdomHTMLInputElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLInputElement * item = (gdom_html_input_element_get_private(thiz));
    g_return_if_fail (item);
    item->select();

}

void
gdom_html_input_element_click (GdomHTMLInputElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLInputElement * item = (gdom_html_input_element_get_private(thiz));
    g_return_if_fail (item);
    item->click();

}

void
gdom_html_input_element_set_selection_range (GdomHTMLInputElement *thiz, glong start, glong end)
{
    g_return_if_fail (thiz);
    WebCore::HTMLInputElement * item = (gdom_html_input_element_get_private(thiz));
    g_return_if_fail (item);
    item->setSelectionRange(start, end);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLInputElement.h"
#include <gdom/GdomHTMLInputElement.h>
#include <gdom/PrivateGdomHTMLInputElement.h>
#include <gdom/PrivateIntGdomHTMLInputElement.h>
#include <gdom/GdomHTMLInputElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLInputElement, gdom_html_input_element, GDOM_TYPE_HTML_ELEMENT);

/*
struct _GdomHTMLInputElementPrivate {
    class WebCore::HTMLInputElement *item;
};

#define GDOM_HTML_INPUT_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_INPUT_ELEMENT, GdomHTMLInputElementPrivate))
*/
G_END_DECLS
WebCore::HTMLInputElement *
gdom_html_input_element_get_private(GdomHTMLInputElement *request)
{
    GDOMHTMLInputElement *res;
    res = static_cast<GDOMHTMLInputElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLInputElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_DEFAULT_VALUE,
    PROP_DEFAULT_CHECKED,
    PROP_FORM,
    PROP_ACCEPT,
    PROP_ACCESS_KEY,
    PROP_ALIGN,
    PROP_ALT,
    PROP_CHECKED,
    PROP_DISABLED,
    PROP_AUTOFOCUS,
    PROP_MAX_LENGTH,
    PROP_NAME,
    PROP_READ_ONLY,
    PROP_SIZE,
    PROP_SRC,
    PROP_TYPE,
    PROP_USE_MAP,
    PROP_VALUE,
    PROP_WILL_VALIDATE,
    PROP_INDETERMINATE,
    PROP_SELECTION_START,
    PROP_SELECTION_END,
    PROP_FILES,
};


static void gdom_html_input_element_finalize(GObject* object)
{
    /*GdomHTMLInputElement* request = GDOM_HTML_INPUT_ELEMENT(object);*/
    /*GdomHTMLInputElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_HTML_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_input_element_parent_class)->finalize(object);
}

static void gdom_html_input_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLInputElement *self = GDOM_HTML_INPUT_ELEMENT(object);
    WebCore::HTMLInputElement * gself = (gdom_html_input_element_get_private(self));
    switch (prop_id) {
     case PROP_DEFAULT_VALUE:
    {
         gself->setDefaultValue(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_DEFAULT_CHECKED:
    {
         gself->setDefaultChecked((g_value_get_boolean (value)) );
         break;
    }
     case PROP_ACCEPT:
    {
         gself->setAccept(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ACCESS_KEY:
    {
         gself->setAccessKey(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ALIGN:
    {
         gself->setAlign(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ALT:
    {
         gself->setAlt(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_CHECKED:
    {
         gself->setChecked((g_value_get_boolean (value)) );
         break;
    }
     case PROP_DISABLED:
    {
         gself->setDisabled((g_value_get_boolean (value)) );
         break;
    }
     case PROP_AUTOFOCUS:
    {
         gself->setAutofocus((g_value_get_boolean (value)) );
         break;
    }
     case PROP_MAX_LENGTH:
    {
         gself->setMaxLength((g_value_get_long (value)) );
         break;
    }
     case PROP_NAME:
    {
         gself->setName(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_READ_ONLY:
    {
         gself->setReadOnly((g_value_get_boolean (value)) );
         break;
    }
     case PROP_SIZE:
    {
         gself->setSize((g_value_get_ulong (value)) );
         break;
    }
     case PROP_SRC:
    {
         gself->setSrc(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_TYPE:
    {
         gself->setType(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_USE_MAP:
    {
         gself->setUseMap(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_VALUE:
    {
         gself->setValue(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_INDETERMINATE:
    {
         gself->setIndeterminate((g_value_get_boolean (value)) );
         break;
    }
     case PROP_SELECTION_START:
    {
         gself->setSelectionStart((g_value_get_long (value)) );
         break;
    }
     case PROP_SELECTION_END:
    {
         gself->setSelectionEnd((g_value_get_long (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_input_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLInputElement *self = GDOM_HTML_INPUT_ELEMENT(object);
    WebCore::HTMLInputElement * gself = (gdom_html_input_element_get_private(self));
    switch (prop_id) {
    case PROP_DEFAULT_VALUE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->defaultValue()));
         break;
    }
    case PROP_DEFAULT_CHECKED:
    {
         g_value_set_boolean (value, gself->defaultChecked());
         break;
    }
    case PROP_FORM:
    {
         PassRefPtr<WebCore::HTMLFormElement> ptr = gself->form();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_ACCEPT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->accept()));
         break;
    }
    case PROP_ACCESS_KEY:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->accessKey()));
         break;
    }
    case PROP_ALIGN:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->align()));
         break;
    }
    case PROP_ALT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->alt()));
         break;
    }
    case PROP_CHECKED:
    {
         g_value_set_boolean (value, gself->checked());
         break;
    }
    case PROP_DISABLED:
    {
         g_value_set_boolean (value, gself->disabled());
         break;
    }
    case PROP_AUTOFOCUS:
    {
         g_value_set_boolean (value, gself->autofocus());
         break;
    }
    case PROP_MAX_LENGTH:
    {
         g_value_set_long (value, gself->maxLength());
         break;
    }
    case PROP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->name()));
         break;
    }
    case PROP_READ_ONLY:
    {
         g_value_set_boolean (value, gself->readOnly());
         break;
    }
    case PROP_SIZE:
    {
         g_value_set_ulong (value, gself->size());
         break;
    }
    case PROP_SRC:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->src()));
         break;
    }
    case PROP_TYPE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->type()));
         break;
    }
    case PROP_USE_MAP:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->useMap()));
         break;
    }
    case PROP_VALUE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->value()));
         break;
    }
    case PROP_WILL_VALIDATE:
    {
         g_value_set_boolean (value, gself->willValidate());
         break;
    }
    case PROP_INDETERMINATE:
    {
         g_value_set_boolean (value, gself->indeterminate());
         break;
    }
    case PROP_SELECTION_START:
    {
         g_value_set_long (value, gself->selectionStart());
         break;
    }
    case PROP_SELECTION_END:
    {
         g_value_set_long (value, gself->selectionEnd());
         break;
    }
    case PROP_FILES:
    {
         PassRefPtr<WebCore::FileList> ptr = gself->files();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_input_element_class_init(GdomHTMLInputElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_input_element_finalize;
    g_klass->set_property = gdom_html_input_element__set_property_fn;
    g_klass->get_property = gdom_html_input_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_DEFAULT_VALUE,
            g_param_spec_string ("default_value", /* name */
                "html_input_element_default_value", /* short description */
                "read-write  gchar * HTMLInputElement.default_value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_DEFAULT_CHECKED,
            g_param_spec_boolean ("default_checked", /* name */
                "html_input_element_default_checked", /* short description */
                "read-write  gboolean HTMLInputElement.default_checked", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FORM,
            g_param_spec_object ("form", /* name */
                "html_input_element_form", /* short description */
                "read-only  GdomHTMLFormElement * HTMLInputElement.form", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_FORM_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ACCEPT,
            g_param_spec_string ("accept", /* name */
                "html_input_element_accept", /* short description */
                "read-write  gchar * HTMLInputElement.accept", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ACCESS_KEY,
            g_param_spec_string ("access_key", /* name */
                "html_input_element_access_key", /* short description */
                "read-write  gchar * HTMLInputElement.access_key", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ALIGN,
            g_param_spec_string ("align", /* name */
                "html_input_element_align", /* short description */
                "read-write  gchar * HTMLInputElement.align", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ALT,
            g_param_spec_string ("alt", /* name */
                "html_input_element_alt", /* short description */
                "read-write  gchar * HTMLInputElement.alt", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CHECKED,
            g_param_spec_boolean ("checked", /* name */
                "html_input_element_checked", /* short description */
                "read-write  gboolean HTMLInputElement.checked", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_DISABLED,
            g_param_spec_boolean ("disabled", /* name */
                "html_input_element_disabled", /* short description */
                "read-write  gboolean HTMLInputElement.disabled", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_AUTOFOCUS,
            g_param_spec_boolean ("autofocus", /* name */
                "html_input_element_autofocus", /* short description */
                "read-write  gboolean HTMLInputElement.autofocus", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_MAX_LENGTH,
            g_param_spec_long ("max_length", /* name */
                "html_input_element_max_length", /* short description */
                "read-write  glong HTMLInputElement.max_length", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_NAME,
            g_param_spec_string ("name", /* name */
                "html_input_element_name", /* short description */
                "read-write  gchar * HTMLInputElement.name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_READ_ONLY,
            g_param_spec_boolean ("read_only", /* name */
                "html_input_element_read_only", /* short description */
                "read-write  gboolean HTMLInputElement.read_only", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SIZE,
            g_param_spec_ulong ("size", /* name */
                "html_input_element_size", /* short description */
                "read-write  gulong HTMLInputElement.size", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SRC,
            g_param_spec_string ("src", /* name */
                "html_input_element_src", /* short description */
                "read-write  gchar * HTMLInputElement.src", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TYPE,
            g_param_spec_string ("type", /* name */
                "html_input_element_type", /* short description */
                "read-write  gchar * HTMLInputElement.type", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_USE_MAP,
            g_param_spec_string ("use_map", /* name */
                "html_input_element_use_map", /* short description */
                "read-write  gchar * HTMLInputElement.use_map", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_VALUE,
            g_param_spec_string ("value", /* name */
                "html_input_element_value", /* short description */
                "read-write  gchar * HTMLInputElement.value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_WILL_VALIDATE,
            g_param_spec_boolean ("will_validate", /* name */
                "html_input_element_will_validate", /* short description */
                "read-only  gboolean HTMLInputElement.will_validate", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_INDETERMINATE,
            g_param_spec_boolean ("indeterminate", /* name */
                "html_input_element_indeterminate", /* short description */
                "read-write  gboolean HTMLInputElement.indeterminate", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SELECTION_START,
            g_param_spec_long ("selection_start", /* name */
                "html_input_element_selection_start", /* short description */
                "read-write  glong HTMLInputElement.selection_start", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SELECTION_END,
            g_param_spec_long ("selection_end", /* name */
                "html_input_element_selection_end", /* short description */
                "read-write  glong HTMLInputElement.selection_end", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FILES,
            g_param_spec_object ("files", /* name */
                "html_input_element_files", /* short description */
                "read-only  GdomFileList * HTMLInputElement.files", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_FILE_LIST, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLInputElementPrivate));*/
}

static void gdom_html_input_element_init(GdomHTMLInputElement* request)
{
    /*GdomHTMLInputElementPrivate* priv = GDOM_HTML_INPUT_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

